@@ -1,5 +1,5 @@
 /* Zutils - Utilities dealing with compressed files
-   Copyright (C) 2009-2021 Antonio Diaz Diaz.
+   Copyright (C) 2009-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Returns true if full_name is a regular file with an enabled extension
+/* Return true if full_name is a regular file with an enabled extension
    or (a link to) a directory. */
 bool test_full_name( const std::string & full_name, const struct stat * stp,
                      const bool follow )
@@ -46,9 +46,9 @@ bool test_full_name( const std::string & full_name, const struct stat * stp,
   }
 
 
-/* Returns in input_filename the next filename, or "." for stdin.
-   ("." was chosen because it is not a valid filename).
-   Sets 'error' to true if a directory fails to open. */
+/* Return in input_filename the next file name, or "." for stdin.
+   ("." was chosen instead of "-" because "." is not a valid file name).
+   Set 'error' to true if a directory fails to open. */
 bool next_filename( std::list< std::string > & filenames,
                     std::string & input_filename, bool & error,
                     const int recursive, const bool ignore_stdin = false,
