@@ -1,5 +1,5 @@
 /* Common code for zcmp and zdiff
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,13 +50,14 @@ int open_other_instream( std::string & name )
   }
 
 
-void parse_format_types2( const std::string & arg, int format_types[2] )
+void parse_format_types2( const std::string & arg, const char * const pn,
+                          int format_types[2] )
   {
   const unsigned i = std::min( arg.find( ',' ), arg.size() );
-  if( i > 0 ) format_types[0] = parse_format_type( arg.substr( 0, i ) );
+  if( i > 0 ) format_types[0] = parse_format_type( arg.substr( 0, i ), pn );
   else format_types[0] = -1;
   if( i + 1 < arg.size() ) format_types[1] =
-    parse_format_type( arg.substr( i + 1 ) );
+    parse_format_type( arg.substr( i + 1 ), pn );
   else format_types[1] = -1;
   }
 
